//-----------------------------------------------------------------------------
// Commande de MSlin Moteur Synchrone linaire  aimants permanents 120
// dsPIC 30F3010
// L. BAGHLI 19/06/2006
// version actuelle : beta : attention composant onduleur L6234 : cmde par bras
//			HallA sur RE0, 2, 4 (PWM low non utiliss)
// 9/10/2006	nouvelle carte Fadi, avec potentiomettre  recouvrement
// 4/12/2006	PWM 16kHz, 20 MIPS
// 1/02/2007	debug LCD contrast et routine
//-----------------------------------------------------------------------------
#include "p30F3010.h"
// 3 modes (non pris en compte l) :
// Pendule	:		autopiotage : cmde 6 secteurs + regul du pendule inverse
//NA MSLin	:		autopiotage : cmde 6 secteurs sans pendule
//NA Vf			:		excitation sinusoidale en V et f independants

#define LCD_SPI_mode

#include "sinus.h"

//Configuration bits
_FOSC(CSW_FSCM_OFF & XT_PLL8);	//10Mhz *8 = 80 MHz /4 = 20 MIPS maxi pour ce pic
_FWDT(WDT_OFF);
_FBORPOR(PBOR_OFF & BORV_27 & PWRT_16 & MCLR_EN);
_FGS(CODE_PROT_OFF);
//-----------------------------------------------------------------------------
//Program Specific Constants
#define FCY 20000000						//Instruction cycle rate (Osc x PLL / 4) = 20 MIPS
#define FPWM 16000						//PWM freq
#define HalfDUTY 	624	// set the pwm half duty = PTPER = 624UL=FCY/FPWM/2-1;
#define FullDUTY HalfDUTY*2			//  car PDCx a une double resolution % PTPER
#define MILLISEC FCY/20000			// 1 mSec delay constant
#define delaisCS 31			// ?? us tps d'envoi de 8 bits sur SPI (pour Fspi=1.25 MHz)
//#define delaisCS 9			// 2 us tps d'envoi de 8 bits sur SPI (pour Fspi=2 MHz)
#define AngleMax 125			// +-Angle max au dela duquel on coupe tt car irratrapable
#define DS_AngleReg 100			// Compteur pour la regulation 100*62.5=6250 us=6.25 ms

#define RunningLED	PORTDbits.RD1				// Output
#define StartPrg		PORTCbits.RC13			// Input			// attention BP2 actif au repos donc j'invers StartPrg parti
#define LCD_CS			PORTCbits.RC14			// Output			// chip select du LCD
// input sondes Hall surles PWM low
#define HallC	PORTEbits.RE0			// la pin PWM1 low  (j'ai invers)
#define HallB	PORTEbits.RE2		// la pin PWM2 low ! 	
#define HallA	PORTEbits.RE4		// la pin PWM3 low ! 
// l'inhib PWM se fait par cavalier sur la carte JP6 ou par EnableL6234
#define EnableL6234	PORTDbits.RD0			// Output
#define DISABLE_FIRING PWMCON1 = 0x0700
#define ENABLE_FIRING  PWMCON1 = 0x0770

void setup_ports(void);
void InitADC10(void);
void DelayNmSec(unsigned int N);
void InitMCPWM(void);

struct {
			unsigned RunMotor		: 	1;
			unsigned SendLCD		:		1;
			unsigned CheckRX		:		1;
			unsigned SendTX			:		1;
			unsigned MSlin			:		1;
			unsigned Pendule		:		1;
			unsigned Vf					:		1;
			unsigned ModeFrontRev	:		1;
			unsigned FrontRev			:		1;
			unsigned CaptIRState	:		1;
			unsigned OldCaptIRState	:		1;
			unsigned OldTogglesAlimState	:		1;
			unsigned unused 		:		4;
		}	Flags;

int RefI;		// RefI (lu sur l'ADC AN2/RB2)
int Vbus;		// lu sur l'ADC AN0/RB0)
int Vref;		// tension de sortie / rapport cyclique
int Vrefmin, Vrefmax;	// limiteur de tension
int Imotor;	// lu sur l'ADC AN1/RB1
int	th_pendule;				// Inclinaison du pendule (lu sur l'ADC AN3/RB3)
int Kp_pendule, Ki_pendule, Kd_pendule, e, Olde, xe_pendule, Offset_pendule, countReg;
int Secteur;
#ifdef LCD_SPI_mode
//variables pour le SPI->LCD
inline void AfficheLCDTable();
inline void LCD_initM();
inline void LCD_initScreen();
inline void DispLCDChar(int hi, int low);
inline void DispLCDOrder(int hi, int low);
inline void trChar();
inline void ConvHexa(int Var, int LCDtablePos);

int LCDlo, LCDhi, LCDupdate, LCDcol, LCDChar, LCDChar_m;
int SeqComm;
#define SeqCommMax 1000
char LCDtable[]="th=0000, Vr=0000H=000 S=0 AcBaCb";
char Str_Ialim[]="AcBaCb";
#define thoffs	3
#define Vroffs	12
#define Hoffs	18
#define Soffs	24
#define Ioffs	26
#endif

/*
unsigned int thetas;
int thetasinc;
int sinthetas, costhetas;
int tmp1, tmp2;
int Vds, Valphas, Vbetas, Vas, Vbs, Vcs;
#define Ktheta	262			// 65536/250
#define SQRT3_2 3547		// sqrt(3)/2 en 4.12
#define ByEs2 800		// en 4.12

int Sequence;
int SeqMaxDelay;
int SeqDelayCounter;
// le nbre de pas ds un sens avant le changement de direction
int FrontRearCounter;
#define FrontRearCounterMax 50
*/


//-----------------------------------------------------------------------------
//	Initialise variables
//-----------------------------------------------------------------------------
void InitVar(void)
{
	U1MODE = 0x0000;	// desable UART + normal pins
#ifdef LCD_SPI_mode
	SeqComm=SeqCommMax;
#endif
	Offset_pendule=55;
//	Kp_pendule=14;		//3,2
//	Ki_pendule=2;
	Kp_pendule=40;		//40
	Ki_pendule=0;
	Kd_pendule=120;		// 80 120
	xe_pendule=0;
	countReg=0;
	Olde=0;
/*
	if (Flags.MAS)
		{
		thetas=0;
		Vas=0;
		Vbs=0;
		Vcs=0;
		Valphas=0;
		Vbetas=0;
		}
	else
		{
		Sequence=0;				// init des variables du deroulement de la seq d'alim des enroulements
		SeqMaxDelay=10;
		SeqDelayCounter=SeqMaxDelay;
		Flags.OldCaptIRState=0;
		Flags.OldTogglesAlimState=0;
		}
	Flags.ModeFrontRev=0;
	Flags.FrontRev=1;
*/
}
//-----------------------------------------------------------------------------
//	Setup ports
//-----------------------------------------------------------------------------
void setup_ports(void)
{
	// Clear All Ports Prior to defining I/O
	ADPCFG = 0xFFF0;				// all PORTB = Digital(1) ;RB0 to RB3 = analog(0)  ie 0000
//	ADPCFG = 0xFFD3;				// all PORTB = Digital(1) ;RB2, RB3 et RB5 = analog(0)  ie F:F:1101:0011
	PORTB=0;	//Initialize LED pin data to off state
	PORTC=0;
	PORTD=0;
	PORTE=0;	
	// Now set pin direction registers
	TRISB = 0xFFFF;		// RB0-5 inputs AN RB0-3, les autres in non utiliss (QEP ou...) 
	TRISC = 0xBFFF;			// RC13 in StartPrg , RC14 out LCD_CS, the rest is input / Unused
	TRISD = 0xFFFC;	// RD0 out EnableL6234, RD1 out RunningLED, le reste (qui n'existe pas) input
	TRISE = 0x00D5;		// RE1, RE3, RE5 are the PWMs -> outputs sauf les low : RE4, RE2, RE0 : Hall input
	TRISF = 0xFFF7;		// RF3= output : SDO1
	LCD_CS = 1;         // Set SPI CS pin du LCD  high : donc non actif
}
//-----------------------------------------------------------------------------
//	Setup the ADC registers for :
//	2 channels conversion (in this case RB2/AN2)
//	PWM trigger starts conversion
//	Pot K_Vf is connected to CH0 and RB2
//	Pot f		 is connected to CH1 and RB3
//	AD interrupt is set and buffer is read in the interrupt 
//-----------------------------------------------------------------------------
void InitADC10(void)
{
 ADCON1 = 0x006F;				// PWM starts conversion, 4 ch simultans, ADC Off for configuring
	if (Flags.Vf)
		ADCON2 = 0x020C;				// 4 simulataneous sample 4 channels, ADC INTerrupt  chaque 4 EOC=400 us car a rempli deja 16 buffer 
	else
		 ADCON2 = 0x0200;				// simulataneous sample 4 channels, ADC INTerrupt  chaque EOC=100 us
// ADCON2 = 0x023C;				// simulataneous sample 4 channels	, ADC interrupt every 16 EOC
 ADCHS	= 0x0003;				// Connect RB3/AN3 as CH0 th_pendule, AN0/RB0 = Ch1 Vbus 
												// AN1/RB1 Ch2 Imotor, AN2/RB2 Ch3 RefI
 ADCON3 = 0x0080;				// Tad = internal RC (4uS)
 IFS0bits.ADIF = 0;			// Adc int flag Off
 IEC0bits.ADIE = 1;			// Adc int On
 ADCON1bits.ADON = 1;			// turn ADC ON
}

//-----------------------------------------------------------------------------
//  InitMCPWM, intializes the Motor PWM as follows:
//  FPWM = 10 khz at 20Mips
//  Independant PWMs
//  Control outputs using OVDCON
//  Init Duty Cycle with a value = 100
//  Set ADC to be triggered by PWM special trigger
//-----------------------------------------------------------------------------
void InitMCPWM(void)
{
	PTPER = HalfDUTY;		// set the pwm period register
	DISABLE_FIRING;			//PWMCON1 = 0x0700;			// disable PWMs
	OVDCON = 0x0000;			// allow control using OVD register
	PDC1 = HalfDUTY;					// init PWM DC 1, 2 and 3 to 800 ou 799= 0 tension (50% duty cycle) car fullDuty=2*PTPER
	PDC2 = HalfDUTY;
	PDC3 = HalfDUTY;
	SEVTCMP = PTPER;		// set ADC to trigeer at ...
//	PWMCON2 = 0x0F00;		// 16 PWM values
	PWMCON2 = 0x0000;		// 1 PWM values
	PTCON = 0x8002;			// start PWM symetrique, however output is enabled by OVDCON which is OFF
}

//-----------------------------------------------------------------------------
//  InitSPI: pour lcd : pas d'interruption, CKP=1 CKE=0, 8 bits
//-----------------------------------------------------------------------------
#ifdef LCD_SPI_mode
void InitSPI()
{
	LCD_CS = 1;         // Set SPI CS pin du LCD  high : donc non actif
//	SPI1CON = 0x003c;                   // Master mode, SCK = Fcy/64 = 230kHz, Input data sampled at 
//	                                    // middle of output time, Data clocked out on rising edge, Clk idle is low
//	SPI1CON = 0x0063;		// Master mode, SCK = Fcy/8 = 2.5 MHz, CKP=1, CKE=0, Clk idle is High, 8 bits
	SPI1CON = 0x007D;		// Master mode, SCK = Fcy/16 = 1.25 MHz, CKP=1, CKE=0, Clk idle is High, 8 bits
	SPI1STAT = 0x8000;	// Enable SPI port
}
#endif

//---------------------------------------------------------------------
//	The ADC interrupt reads the demand pot value. 
// tt est synchrone %  cette int de 100 us
//---------------------------------------------------------------------
void __attribute__((__interrupt__)) _ADCInterrupt (void)
{
// InterruptLED=1;	
	IFS0bits.ADIF = 0;	
	Vbus=ADCBUF1>>1;			
	Imotor=ADCBUF2>>1;		// 1024*2=2048	
	RefI=ADCBUF3;			// 0  1023
	if (RefI>HalfDUTY)	RefI=HalfDUTY;	// jusqu 624
	Vrefmax=FullDUTY-RefI;		// si RefI=0 alors pleine puissance autorise
	Vrefmin=RefI;
//		Vref=ADCBUF3;				// max 1023 > FullDUTY=922
//		if (Vref>FullDUTY)	Vref=FullDUTY;
//// reg P
//		Vref=ADCBUF3;				// max 1023 > FullDUTY=922
//		Vref=(512-Vref+25)*8 +HalfDUTY;
//		if (Vref<0)	Vref=0;
//		if (Vref>FullDUTY)	Vref=FullDUTY;

// reg PI
	th_pendule=ADCBUF0;				// max 1023 > FullDUTY=922
//		th_pendule=RefI<<1;	on ne peut plus l'avoir	// si on veut le potar de la carte

	if (Flags.RunMotor ==0)	Offset_pendule=th_pendule-512;	// mode arret onduleur, alors acquisition de la position verticale du pendule ie offset
// excute toutes les DS_AngleReg(100) * fois
	if ( ++countReg==DS_AngleReg )
		{
		countReg=0;
		// regul angle
		e=-(512-th_pendule+Offset_pendule);		// je l'inverse pour la version V2
		if (e>AngleMax || e<-AngleMax)	Vref=HalfDUTY;		// angle trop grand pour pouvoir rguler alors on coupe
			else	{
					Vref =Kp_pendule*e + xe_pendule + Kd_pendule*(e-Olde) + HalfDUTY;
//						if ( Vref>Vrefmin && Vref<Vrefmax )	
							xe_pendule += Ki_pendule*e;
				if (xe_pendule<-FullDUTY)		xe_pendule=-FullDUTY;
				if (xe_pendule>FullDUTY)		xe_pendule=FullDUTY;
						if (Vref<Vrefmin)	Vref=Vrefmin;
						if (Vref>Vrefmax)	Vref=Vrefmax;
						Olde=e;
						}
		}

// determiner Secteur
	// lit les sondes Hall et determine la config  alimenter
	Secteur=HallA | HallB<<1 | HallC<<2;		
	//en fonction du secteur, determine les refIabcs et les PDCx
	switch (Secteur)
		{
		case 1 :   // FALLING3
						PDC1 = Vref;	PDC2 = HalfDUTY;		PDC3 = FullDUTY-Vref;					
#ifdef LCD_SPI_mode
						sprintf(Str_Ialim,"AC ac ");
#endif
						break;
		case 2 :   // FALLING1
						PDC1 = FullDUTY-Vref;	PDC2 = Vref;		PDC3 = HalfDUTY;					
#ifdef LCD_SPI_mode
						sprintf(Str_Ialim,"a BA b");
#endif
						break;
		case 3 :   // RISING2
						PDC1 = HalfDUTY;	PDC2 = Vref;		PDC3 = FullDUTY-Vref;					
#ifdef LCD_SPI_mode
						sprintf(Str_Ialim," CB cb");
#endif
						break;
		case 4 :   // FALLING2
						PDC1 = HalfDUTY;	PDC2 = FullDUTY-Vref;		PDC3 = Vref;					
#ifdef LCD_SPI_mode
						sprintf(Str_Ialim," cb CB");
#endif
						break;
		case 5 :   // RISING1
						PDC1 = Vref;	PDC2 = FullDUTY-Vref;		PDC3 = HalfDUTY;					
#ifdef LCD_SPI_mode
						sprintf(Str_Ialim,"A ba B");
#endif
						break;
		case 6 :   // RISING3
						PDC1 = FullDUTY-Vref;	PDC2 = HalfDUTY;		PDC3 = Vref;					
#ifdef LCD_SPI_mode
						sprintf(Str_Ialim,"ac AC ");
#endif
						break;
		default : // cas impossible : on coupe tt
						PDC1 = HalfDUTY;	PDC2 = HalfDUTY;		PDC3 = HalfDUTY;					
#ifdef LCD_SPI_mode
						sprintf(Str_Ialim,"-none-");
#endif

		}
#ifdef LCD_SPI_mode
//comme elle prend du temps c'est soit afficher 1 char,soit un ordre  l'LCD
//affiche la LCD table si besoin
	switch (LCDupdate)
		{
		case 1 :	AfficheLCDTable(); break;
		case 2 :	LCDupdate=0;
							DispLCDOrder( 0x00, 0x40);					//Retour curseur  l'origine
							break;
		default:	// si zero alors demande la MAJ des I1 et I2
							// execute toutes les (1000+32+1)*125 us
							if (--SeqComm>=0)		break;
							SeqComm=SeqCommMax;
							LCDupdate=1;		// Cursor home
							ConvHexa(th_pendule, thoffs);	// th en Hexa
//							ConvHexa(xe_pendule, thoffs);	// th en Hexa
							ConvHexa(Vref, Vroffs);				// Vref en Hexa
							LCDtable[Hoffs+2]=HallA+0x30;
							LCDtable[Hoffs+1]=HallB+0x30;
							LCDtable[Hoffs  ]=HallC+0x30;
							LCDtable[Soffs]=Secteur+0x30;
							LCDtable[Ioffs  ]=Str_Ialim[0];
							LCDtable[Ioffs+1]=Str_Ialim[1];
							LCDtable[Ioffs+2]=Str_Ialim[2];
							LCDtable[Ioffs+3]=Str_Ialim[3];
							LCDtable[Ioffs+4]=Str_Ialim[4];
							LCDtable[Ioffs+5]=Str_Ialim[5];
		}
#endif
//	InterruptLED=0;	
}

#ifdef LCD_SPI_mode
//----------------------------------------------------
// initialisation de l'cran LCD
//----------------------------------------------------
inline void LCD_initScreen()
{
			DispLCDOrder( 0xC0, 0x00);			// fonc 8 bits
			DelayNmSec(4);

			DispLCDOrder( 0x00, 0x80);			// clear LCD, 1.53 ms
			DelayNmSec(4);

			DispLCDOrder( 0x00, 0x60);					// increment, 1.53 ms
			DelayNmSec(4);

			DispLCDOrder( 0x00, 0x30);					// display on, cursor Off
			DelayNmSec(2);

			LCDupdate=1;			// doit MAJ la table sur l'Ecran LCD
			LCDcol=0;
}
//----------------------------------------------------
// Routine trChar : transforme le char LCDChar en LCDChar_m (pour le LCD)
//----------------------------------------------------
inline void trChar()
{
unsigned int tmp;
	LCDChar_m=0x00;
	for (tmp=8; tmp>0; tmp--)
		{
		LCDChar_m=LCDChar_m<<1 | (LCDChar & 1);
		LCDChar=LCDChar>>1;
		}
	LCDlo=LCDChar_m & 0xF0;
	LCDhi=LCDChar_m<<4 & 0xF0;
}
//----------------------------------------------------
// Routine d'envoi de donnees sur SPI (pour LCD)
// met  jour la LCDtable sur l'cran
//----------------------------------------------------
inline void AfficheLCDTable()
{
	LCDChar=LCDtable[LCDcol];
// essai debug force le LCDchar  un A
//	LCDChar=0x41;
	trChar();
	DispLCDChar( LCDhi, LCDlo);	// sortir le Char actuel et inutile d'attendre car routine ttes les 200 us
	if (++LCDcol>=32)	{
										LCDcol=0;
										LCDupdate=2;		// demande de remise  la 1er ligne du curseur
										}
}
//----------------------------------------------------
// Convertit un "Word" hexa en "4 chars Hexadcimaux"
// et les sort sur la table en memoire
//----------------------------------------------------
inline void ConvHexa(int Var, int LCDtablePos)
{  
int tmp;
			tmp=Var & 0x000F;
			if (tmp<=9)		tmp+=0x30;
					else			tmp+=0x37;
			LCDtable[LCDtablePos+3]=tmp;

			tmp=Var>>4 & 0x000F;
			if (tmp<=9)		tmp+=0x30;
					else			tmp+=0x37;
			LCDtable[LCDtablePos+2]=tmp;

			tmp=Var>>8 & 0x000F;
			if (tmp<=9)		tmp+=0x30;
					else			tmp+=0x37;
			LCDtable[LCDtablePos+1]=tmp;

			tmp=Var>>12 & 0x000F;
			if (tmp<=9)		tmp+=0x30;
					else			tmp+=0x37;
			LCDtable[LCDtablePos]=tmp;
}
//----------------------------------------------------
// Envoi 3 bytes pour Afficher : 1=> ordre, 2 => Char
//----------------------------------------------------
inline void DispLCDChar(int hi, int low)
{
	unsigned int dummy, j;
	LCD_CS=1;								// Desacive le LCD via le /CS
	SPI1STATbits.SPIROV = 0;            // Clear overflow flag
	dummy = SPI1BUF;                    // Read buffer to avoid overflow
	LCD_CS=0;								// Active le LCD via le /CS
	SPI1BUF=0x00FA;		// Write to the output buffer start Char
  while (SPI1STATbits.SPITBF == 1);   // Wait for transmission to complete

	SPI1BUF=low;
  while (SPI1STATbits.SPITBF == 1);   // Wait for transmission to complete

	SPI1BUF=hi;
  while (SPI1STATbits.SPITBF == 1);   // Wait for transmission to complete
 	for(j=0;j < delaisCS;j++);	// delai avant CS car ds que le buffer est transmis alors ils libere le SPITBF 
	LCD_CS=1;								// Desacive le LCD via le /CS
	// Pas besoin d'attendre, car l'appel  cette routine ne se fait au mieux ttes les 100 us
	// donc le LCD a tt le tps d'executer
}
//-----------------------------------------------------------------------------
inline void DispLCDOrder(int hi, int low)
{
	unsigned int dummy, j;
	LCD_CS=1;								// Desacive le LCD via le /CS
	SPI1STATbits.SPIROV = 0;            // Clear overflow flag
	dummy = SPI1BUF;                    // Read buffer to avoid overflow
	LCD_CS=0;								// Active le LCD via le /CS
	SPI1BUF=0x00F8;					// Write to the output buffer start Order
  while (SPI1STATbits.SPITBF == 1);   // Wait for transmission to complete

	SPI1BUF=low;
  while (SPI1STATbits.SPITBF == 1);   // Wait for transmission to complete

	SPI1BUF=hi;
  while (SPI1STATbits.SPITBF == 1);   // Wait for transmission to complete
 	for(j=0;j < delaisCS;j++);	// delai avant CS car ds que le buffer est transmis alors ils libere le SPITBF 
	LCD_CS=1;								// Desacive le LCD via le /CS
	// Pas besoin d'attendre, car l'appel  cette routine ne se fait au mieux ttes les 100 us
	// donc le LCD a tt le tps d'executer
}
#endif


//-----------------------------------------------------------------------------
//Main routine
int main(void)
{
	setup_ports();
#ifdef LCD_SPI_mode
	DelayNmSec(100);		// wait 0.1 s for LCD to stabilise its power
	InitSPI();
	SeqComm=SeqCommMax;
	LCD_initScreen();
#endif
	InitVar();
	InitMCPWM();
	InitADC10();
	RunningLED=0;	
//  ^^^^ dja fait ds le setup port, mais introduit un temps d'attente

	// check user choice : MSlin, Pendul ou Vf
	if (StartPrg) {
								Flags.Vf=0;		//alors mode MSlin sans pendule
								Flags.Pendule=0;
								Flags.MSlin=1;
								DelayNmSec(1000);		// wait 1 s
								RunningLED = 1;
								if (StartPrg)	// still pressed  alors Vf
										{
										Flags.Vf=1;
										Flags.MSlin=0;
										Flags.Pendule=0;
										}
								while (StartPrg)	DelayNmSec(10);		// wait 'till key is released
								}
			else			{ // par defaut : mode MSlin + regul Pendule
								Flags.Vf=0;
								Flags.Pendule=1;
								Flags.MSlin=0;
								}

	RunningLED = 0;
	while (StartPrg)	DelayNmSec(10);		// wait 'till key is released
	while(1)
		{	
		while (!StartPrg);									// wait for start key hit
		while (StartPrg)	DelayNmSec(10);		// wait 'till key is released
		RunningLED=1;						// LED run = On
			// lit la position au dbut et det le secteur
			// lit les sondes Hall et determine la config  alimenter
			//				Secteur=HallA | HallB<<1 | HallC<<2;		
			//				OVDCON = StateTable[Secteur];	// Load the overide control register
		PDC1=HalfDUTY; PDC2=HalfDUTY; PDC3=HalfDUTY;			// init sans rien, au bout de 100us a change
		OVDCON = 0xFFFF;	// Cmde MLI, no effect of OVDCON
		ENABLE_FIRING;				//PWMCON1= 0x0770;					// enable PWM outputs (on touche pas aux LOW qui sont en input)
#ifdef LCD_SPI_mode
				LCDtable[Ioffs-1]='*';
#endif
		EnableL6234=1;						// enable L6234 
		Flags.RunMotor = 1;					// set running flag
		DelayNmSec(100);
		while (Flags.RunMotor)				// while motor is running
			{
			if (StartPrg)							// if S2 is pressed, stop
				{
				EnableL6234=0;						// disable L6234
#ifdef LCD_SPI_mode
				LCDtable[Ioffs-1]=' ';
#endif
				DISABLE_FIRING;			// disable PWM outputs
				PDC1=HalfDUTY; PDC2=HalfDUTY; PDC3=HalfDUTY;	
	  		OVDCON = 0x0000;			// overide PWM High.
				Flags.RunMotor = 0;			// reset run flag
				RunningLED=0;						// LED run = Off
				while (StartPrg)	DelayNmSec(10);		// wait 'till key is released
				}

			if (Flags.SendLCD)
				{
				//SendLCD();		// send present fs serially
				Flags.SendLCD = 0;		// clear flag 
				}
			} //end of while (Flags.RunMotor)
		}	// end of while (1)
}	// end of main

//=============================================================================
//Error traps
//-----------------------------------------------------------------------------
//Oscillator Fail Error trap routine
void _ISR _OscillatorFail(void)
{	
	//Flags.RunMotor reste  1, pour distinguer defaut, de fonct. normal.   // Flags.RunMotor = 0;			// reset run flag
	EnableL6234=0;						// disable L6234
	RunningLED=0;
#ifdef LCD_SPI_mode
	LCDtable[Ioffs-1]='O';
#endif
	while(1); //Wait forever
}
//-----------------------------------------------------------------------------
//Address Error trap routine
void _ISR _AddressError(void)
{
	//Flags.RunMotor reste  1, pour distinguer defaut, de fonct. normal.   // Flags.RunMotor = 0;			// reset run flag
	EnableL6234=0;						// disable L6234
	RunningLED=0;
#ifdef LCD_SPI_mode
	LCDtable[Ioffs-1]='A';
#endif
	while(1); //Wait forever
}
//-----------------------------------------------------------------------------
//Stack Error trap routine
void _ISR _StackError(void)
{
	//Flags.RunMotor reste  1, pour distinguer defaut, de fonct. normal.   // Flags.RunMotor = 0;			// reset run flag
	EnableL6234=0;						// disable L6234
	RunningLED=0;
#ifdef LCD_SPI_mode
	LCDtable[Ioffs-1]='S';
#endif
	while(1); //Wait forever
}
//-----------------------------------------------------------------------------
//Math (Arithmetic) Error trap routine
void _ISR _MathError(void)
{
	//Flags.RunMotor reste  1, pour distinguer defaut, de fonct. normal.   // Flags.RunMotor = 0;			// reset run flag
	EnableL6234=0;						// disable L6234
	RunningLED=0;
#ifdef LCD_SPI_mode
	LCDtable[Ioffs-1]='M';
#endif
	while(1); //Wait forever
}
//---------------------------------------------------------------------
// This is a generic 1ms delay routine to give a 1mS to 65.5 Seconds delay
// For N = 1 the delay is 1 mS, for N = 65535 the delay is 65,535 mS. 
// Note that FCY is used in the computation.  Please make the necessary
// Changes(PLLx4 or PLLx8 etc) to compute the right FCY as in the define
// statement above.
//---------------------------------------------------------------------
void DelayNmSec(unsigned int N)
{
unsigned int j;
while(N--)
 	for(j=0;j < MILLISEC;j++);
}
//---------------------------------------------------------------------
